# grunt-ejs-render

> Render ejs templates with custom data and helpers

This plugin provides ejs static rendering to enhance static file development.

Aside from default [ejs features](https://github.com/visionmedia/ejs#features) it provides:

* Lo-Dash/underscore functions (http://lodash.com/docs)
* Lo-Dash/underscore templates powered view partials (http://lodash.com/docs#template)
* an easy way to define custom _per task_ helpers 

## Getting Started
This plugin requires Grunt `~0.4.1`

If you haven't used [Grunt](http://gruntjs.com/) before, be sure to check out the [Getting Started](http://gruntjs.com/getting-started) guide, as it explains how to create a [Gruntfile](http://gruntjs.com/sample-gruntfile) as well as install and use Grunt plugins. Once you're familiar with that process, you may install this plugin with this command:

```shell
npm install grunt-ejs-render --save-dev
```

Once the plugin has been installed, it may be enabled inside your Gruntfile with this line of JavaScript:

```js
grunt.loadNpmTasks('grunt-ejs-render');
```

## The "render" task

### Overview
In your project's Gruntfile, add a section named `render` to the data object passed into `grunt.initConfig()`.

```js
grunt.initConfig({
  render: {
    options: {
      // Task-specific options go here.
    },
    your_target: {
      // Target-specific file lists and/or options go here.
    }
  }
})
```

### Options

#### options._
Type: `Object`
Default value: `grunt.util._`

A reference to a [Lo-Dash](http://lodash.com) build. Defaults to the one shipped with Grunt.


```js

//load a Backbone build of Lo-Dash
var bb_ = require('./customlibs/lodash/lodash-backbone.js');

grunt.initConfig({
  render: {
    options: {
     '_' : bb_ 
    }
    //...
  }
})
```



Inside a template you may access Lo-Dash functions from `_`:

```
<p><%= _.first(['orange', 'lemon', 'apple']) %></p>
<!-- oputputs <p>orange</p> -->
```


#### options.data
Type: `Object|String`
Default value: `null`

An object containing dynamic data to be passed to templates. You may also pass a JSON filepath as a string.

```js
grunt.initConfig({
  render: {
    first_target: {
      data: 'path/to/file.json'
    },
    second_target: {
      data: { 'prop': 'my test'}
    }
  }
})
```

To access datas from inside a template use `data.` namespace:

```
<p><%= data.prop %></p>
```


#### options.templates
Type: `Mixed`
Default value: `[]`

An [array of files](http://gruntjs.com/configuring-tasks#files) of [Lo-Dash templates](http://lodash.com/docs#template) to be used inside a main template file. May be useful to reuse client side templates to render a static file.

Compiled templates will be indexed by their filename without extension, and are accessible with the `helpers.template` helper method

Template configuration

```js
grunt.initConfig({
  render: {
    options: {
      templates: ['templates/*.tpl']
    }
  }
})
```

Usage

```
<!-- templates/list.tpl -->

<% fruits.forEach(function (fruit) { %>
  <li><%= fruit %></li>
<% }); %>
```

```
<!-- main.html -->

<p><%= helpers.template('list', {fruits: ['orange', 'lemon', 'apple']}) %></p>
```

#### options.helpers
Type: `Object`
Default value: `{}`

Hash of custom methods for usage inside a template.

Default helpers are:

* `template('templatename', dataObject)`: executes a precompiled Lo-Dash template (if available) with provided data object
* `getMTime('filepath')`: returns the last modified time (as unix timestamp) of the passed in file. `filepath` is relative to `Gruntfile.js`

Helpers configuration

```js
grunt.initConfig({
  render: {
    helpers: {
      //set a custom helper
      timestamp: function () { return new Date().getTime(); }
    }
  }
})
```

Usage inside template

```
<!-- cache bursting -->
<script src="/lib/script.js?v=<%= helpers.getMTime('/lib/script.js') %>"></script>

<!-- lo dash template -->
<p><%= helpers.template('list', {fruits: ['orange', 'lemon', 'apple']}) %></p>

<!-- custom helper -->
<p>build timestamp: <%= helpers.timestamp() %></p>
```


### Usage Examples

#### Default Options
To process a file with ejs just pass it to the `files` array:

```js
grunt.initConfig({
  render: {
    options: {},
    files: {
      'dest/index.html': ['src/index.html']
    }
  }
});
```

#### Custom Options
You may provide custom options:

```js
grunt.initConfig({
  render: {
    options: {
      data: 'data/fruits.json'
      helpers: {
        timestamp: function () { return new Date().getTime(); }
      },
      templates: ['templates/*.tpl']
    },
    files: {
      'dest/fruits.html': ['src/fruits.html']
    }
  }
});
```

## Contributing
In lieu of a formal styleguide, take care to maintain the existing coding style. Add unit tests for any new or changed functionality. Lint and test your code using [Grunt](http://gruntjs.com/).

## Release History


0.1.0 - Initial release
0.1.1 - Better Docs
